/* 1- Se cuenta con un archivo directo llamado ALUMNOS.DAT con informacin de alumnos de una  universidad. El archivo se abre para lectura y se indexa por medio de su clave primaria (#legajo). El ndice se mantiene en memoria principal. Se ofrece la posibilidad al usuario de buscar alumnos por legajo. La bsqueda binaria se aplica sobre el ndice y se accede en forma directa (seek) al archivo para recuperar el registro. Se asume que el archivo est ordenado de manera ascendente por su clave primaria (#legajo). Si ste no fuese el caso, el ndice debera ordenarse antes de aplicar una bsqueda binaria sobre l. /* Aclaracin: no se provee el archivo ALUMNOS.DAT (debido a que es binario). Deber  ser generado antes de la ejecucin de este programa).*/
/* Solucin en lenguaje C */

#include <stdio.h>

#define TAMANIO_INDICE 5

typedef struct {
	int dia, mes, anio;
} t_fecha;

typedef struct {
	long legajo;
	char nomyap[30+1];
	char documento[10+1];
	char direccion[40+1];
	t_fecha fecha_nac;
} t_alumno;

/* Estructura de cada entrada en el ndice */
typedef struct {
	long clave; /* Clave de acceso (legajo) */
	long posicion; /* Posicin del registro en el archivo */
} t_entrada_indice;

typedef t_entrada_indice t_indice[TAMANIO_INDICE];

typedef enum {FALSO, VERDADERO} t_bool;

void indexar(FILE* arch, t_indice indice);
t_bool buscar(FILE* arch, t_indice indice, long legajo, t_alumno* alumno);
void mostrar_datos(t_alumno alumno);

int main()
{
	FILE*		alumnos;
	t_indice	indice;
	long		legajo;
	t_alumno	alumno;
	t_bool		encontrado;
	int		i;
	if ((alumnos=fopen("alumnos.dat","rb")) == NULL)
	{
		printf("No se puede abrir el archivo.\n");
		return 1;
	}

	/* Indexacin del archivo */
	indexar(alumnos, indice);

	do
	{
		printf("Ingrese un legajo a buscar (-1 para terminar): ");
		scanf("%ld", &legajo);

		if (legajo != -1)
		{
		/* Bsqueda binaria sobre el ndice */
		encontrado = buscar(alumnos, indice, legajo, &alumno);
		if (encontrado)
			mostrar_datos(alumno);
			else
			printf("El legajo %ld no se encuentra en el archivo.\n", legajo);
		}
		printf("\n");

	} while (legajo != -1);
	fclose(alumnos);
	return 0;
}

void indexar(FILE* arch, t_indice indice)
{
	t_alumno alumno;
	int posicion, i = 0;
	rewind(arch);
	do
	{
		posicion = ftell(arch); /* Posicin actual del puntero a registro */
		fread(&alumno, sizeof(t_alumno), 1, arch);
		if (!feof(arch))
		{
			indice[i].clave = alumno.legajo;
			indice[i].posicion = posicion;
			i++;
		}
		} while (!feof(arch));
}
t_bool buscar(FILE* arch, t_indice indice, long legajo, t_alumno* alumno)
{
	long inferior, superior, centro;
	t_bool encontrado = FALSO;

	inferior = 0;
	superior = TAMANIO_INDICE-1;

	while ( (inferior <= superior) && !encontrado )
	{
		centro = (inferior + superior) / 2; /* Divisin entera */
		if (indice[centro].clave == legajo)
		{
			encontrado = VERDADERO;
		}
		else
		{
			if (legajo < indice[centro].clave)
			superior = centro - 1; /* Buscar en la primera mitad */
			else
			inferior = centro + 1; /* Buscar en la segunda mitad */
		}
	}

	if (encontrado)
	{
		fseek(arch, indice[centro].posicion, SEEK_SET);
		fread(alumno, sizeof(t_alumno), 1, arch);
	}

	return encontrado;
}
void mostrar_datos(t_alumno alumno)
{

	printf("Legajo: %ld\n", alumno.legajo);
	printf("Nombre y apellido: %s\n", alumno.nomyap);
	printf("Documento de identidad: %s\n", alumno.documento);
	printf("Direccin: %s\n", alumno.direccion);
	printf("Fecha de nacimiento: %d/%d/%d\n",alumno.fecha_nac.dia,alumno.fecha_nac.mes,alumno.fecha_nac.anio);
}